import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A Mover is an actor that also has 'move' and 'turn' ability. Both moving and turning 
 * are relative to its current position. When moving, the Mover will move in the direction 
 * it is currently facing.
 * 
 * Both 'move' and 'turn' methods are available with or without parameters.
 * 
 * The 'Mover' class is a subclass of Actor. It can be used by creating subclasses, or
 * copied into scenarios and edited inline.
 * 
 * The initial direction is to the right. Thus, this class works best with images that
 * face right when not rotated.
 * 
 * This class can also check whether we are close to the edge of the world.
 * 
 * @author Michael Kolling
 * @version 1.0 (July 2007)
 */
public class Mover extends Actor
{
    private static final double WALKING_SPEED = 5.0;
    private Vector speed = new Vector();
    
    private double x = 0;
    private double y = 0;
    
    /**
     * Turn 90 degrees to the right (clockwise).
     */
    public void turn()
    {
        turn(90);
    }
    

    /**
     * Turn 'angle' degrees towards the right (clockwise).
     */
    public void turn(int angle)
    {
        setRotation(getRotation() + angle);
    }
    

    /**
     * Move a bit forward in the current direction.
     */
    /**
     * Move forward one step.
     */
    public void move() {
        x = x + speed.getX();
        y = y + speed.getY();
        if(x >= getWorld().getWidth()) {
            x = 0;
        }
        if(x < 0) {
            x = getWorld().getWidth() - 1;
        }
        if(y >= getWorld().getHeight()) {
        }
        if(y < 0) {
            y = getWorld().getHeight() - 1;
        }
        setLocation(x, y);
    }
    
    /**
     * Test if we are close to one of the edges of the world. Return true is we are.
     */
    public boolean atWorldEdge()
    {
        if(getX() < 10 || getX() > getWorld().getWidth() - 10)
            return true;
        if(getY() < 10 || getY() > getWorld().getHeight() - 10)
            return true;
        else
            return false;
    }
    
    public void setLocation(double x, double y) {
        this.x = x;
        this.y = y;
        super.setLocation((int) x, (int) y);
    }
    
    public void setLocation(int x, int y) {
        setLocation((double) x, (double) y);
    }

    /**
     * Increase the speed with the given vector.
     */
    public void increaseSpeed(Vector s) {
        speed.add(s);
    }
    
    public void subSpeed(Vector s) {
        speed.sub(s);
    }
    
    /**
     * Return the current speed.
     */
    public Vector getSpeed() {
        return speed;
    }

    /**
     * Get the sea object itself.
     */
    protected final Sea getSea() {
        return (Sea)getWorld();
    }

    public void bounceSonar()
    {
        getSpeed().reset();
        if(getRotation()>=180)
        {
        getSpeed().reset();
        increaseSpeed(new Vector(getRotation()-180, 10));
        }else if(getRotation()<180)
        {
        getSpeed().reset();
        increaseSpeed(new Vector(getRotation()+180, 10));
        }
        move();
    }
    
    public void bounceBubble()
    {
        getSpeed().reset();
        if(getRotation()>=180)
        {
        getSpeed().reset();
        increaseSpeed(new Vector(getRotation()-180, 2));
        }else if(getRotation()<180)
        {
        getSpeed().reset();
        increaseSpeed(new Vector(getRotation()+180, 2));
        }
        move();
    }
   
}
